@@ -211,49 +211,210 @@ class BackupEngine {
     }
     
     public function downloadBackup($backupId, $user) {
+        // Log download initiation
+        error_log("AUDIT: Download initiated - User: {$user}, Backup ID: {$backupId}, IP: " . ($_SERVER['REMOTE_ADDR'] ?? 'unknown'));
+
         $history = new BackupHistory($this->db, $backupId);
-        
+
         if (!$history->canAccess($user)) {
+            error_log("SECURITY: User '{$user}' attempted to access backup {$backupId} (access denied)");
             throw new Exception("Access denied to backup");
         }
-        
+
         $data = $history->getData();
         if ($data['status'] !== 'completed') {
+            error_log("Download failed - Backup not completed - User: {$user}, Backup ID: {$backupId}, Status: {$data['status']}");
             throw new Exception("Backup is not available for download");
         }
-        
+
+        // Secondary validation: verify backup target access
         $target = new BackupTarget($this->db, $data['target_id']);
-        $storage = new BackupStorage($target->getCredentials());
-        
-        // Get backup info
-        $info = $storage->getFileInfo($data['backup_path']);
-        if (!$info['success'] || !$info['exists']) {
-            throw new Exception("Backup file not found in storage");
+        if (!$target->canAccess($user)) {
+            error_log("SECURITY: User '{$user}' lacks access to backup target {$data['target_id']} for backup {$backupId}");
+            throw new Exception("Access denied to backup");
         }
-        
-        // Set headers for download
-        // Use the actual filename from the backup path
+
+        // Validate upload_status field (whitelist validation)
+        $uploadStatus = $data['upload_status'] ?? null;
+        $validStatuses = ['pending', 'uploading', 'uploaded', 'failed', 'skipped'];
+        if ($uploadStatus !== null && !in_array($uploadStatus, $validStatuses, true)) {
+            error_log("SECURITY: Invalid upload_status in database - Backup ID: {$backupId}, Status: {$uploadStatus}");
+            throw new Exception("Backup file is not currently available for download");
+        }
+
+        // Check if backup is available in remote storage
+        $useLocalFallback = false;
+
+        // Use local fallback if:
+        // 1. Upload hasn't been attempted yet (pending/null)
+        // 2. Upload failed
+        // 3. Remote file is not accessible
+        if ($uploadStatus === null || $uploadStatus === 'pending' || $uploadStatus === 'uploading' || $uploadStatus === 'failed') {
+            $useLocalFallback = true;
+        } else if ($uploadStatus === 'uploaded') {
+            // Check if remote file actually exists
+            $storage = new BackupStorage($target->getCredentials());
+            $info = $storage->getFileInfo($data['backup_path']);
+
+            if (!$info['success'] || !$info['exists']) {
+                // Remote file not found, try local fallback
+                $useLocalFallback = true;
+            }
+        }
+
+        // Sanitize filename for Content-Disposition header
         $filename = basename($data['backup_path']);
-        
+        $filename = preg_replace('/[^\w\.\-]/', '_', $filename);
+        if (strlen($filename) > 255) {
+            $filename = substr($filename, 0, 255);
+        }
+        $encodedFilename = rawurlencode($filename);
+
         // Clean any output buffers to ensure no extra content
         while (ob_get_level()) {
             ob_end_clean();
         }
-        
-        header('Content-Type: application/octet-stream');
-        header('Content-Disposition: attachment; filename="' . $filename . '"');
-        header('Content-Length: ' . $info['size']);
-        header('Cache-Control: no-cache, must-revalidate');
-        header('Pragma: no-cache');
-        
-        
-        // Stream the file directly to output
-        $result = $storage->streamFile($data['backup_path']);
-        
-        if (!$result['success']) {
-            throw new Exception("Failed to stream backup: " . $result['error']);
+
+        if ($useLocalFallback) {
+            // Attempt to serve from local storage
+            $localPath = $data['local_path'] ?? null;
+
+            // Validate local_path type
+            if ($localPath !== null && !is_string($localPath)) {
+                error_log("SECURITY: Invalid local_path type - Backup ID: {$backupId}");
+                throw new Exception("Backup file is not currently available for download");
+            }
+
+            if (!$localPath) {
+                error_log("Download failed - No local path - User: {$user}, Backup ID: {$backupId}, Upload status: {$uploadStatus}");
+                throw new Exception("Backup file is not currently available for download");
+            }
+
+            // CRITICAL: Validate path to prevent directory traversal attacks
+            $allowedPaths = [
+                '/docker/backups/temp/',
+                '/docker/backups/ready/',
+                '/docker/backups/failed/'
+            ];
+
+            $realPath = realpath($localPath);
+            if ($realPath === false) {
+                error_log("SECURITY: Invalid backup file path - Backup ID: {$backupId}, Path: {$localPath}");
+                throw new Exception("Backup file is not currently available for download");
+            }
+
+            $isAllowed = false;
+            foreach ($allowedPaths as $allowedPath) {
+                $realAllowedPath = realpath($allowedPath);
+                if ($realAllowedPath && strpos($realPath, $realAllowedPath) === 0) {
+                    $isAllowed = true;
+                    break;
+                }
+            }
+
+            if (!$isAllowed) {
+                error_log("SECURITY: Attempted access to unauthorized path - Backup ID: {$backupId}, Path: {$localPath}, Real: {$realPath}");
+                throw new Exception("Backup file is not currently available for download");
+            }
+
+            // Check if local file was deleted
+            if ($data['local_deleted_at'] !== null) {
+                error_log("Download failed - Local file deleted - User: {$user}, Backup ID: {$backupId}, Deleted at: {$data['local_deleted_at']}");
+                throw new Exception("Backup file is not currently available for download");
+            }
+
+            // Open file atomically to avoid race conditions
+            $fp = @fopen($realPath, 'rb');
+            if (!$fp) {
+                error_log("Download failed - Cannot open local file - User: {$user}, Backup ID: {$backupId}, Path: {$realPath}");
+                throw new Exception("Backup file is not currently available for download");
+            }
+
+            // Get file stats from open handle
+            $stats = fstat($fp);
+            if ($stats === false) {
+                fclose($fp);
+                error_log("Download failed - Cannot stat file - User: {$user}, Backup ID: {$backupId}");
+                throw new Exception("Backup file is not currently available for download");
+            }
+
+            // Verify it's a regular file (not symlink or device)
+            if (!($stats['mode'] & 0100000)) { // S_IFREG check
+                fclose($fp);
+                error_log("SECURITY: Attempted to download non-regular file - Backup ID: {$backupId}, Path: {$realPath}");
+                throw new Exception("Backup file is not currently available for download");
+            }
+
+            $fileSize = $stats['size'];
+
+            // Log successful local file access
+            error_log("AUDIT: Serving backup from local storage - User: {$user}, Backup ID: {$backupId}, Path: {$realPath}, Size: {$fileSize}");
+
+            // Set security headers
+            header('Content-Type: application/octet-stream');
+            header("Content-Disposition: attachment; filename=\"{$filename}\"; filename*=UTF-8''{$encodedFilename}");
+            header('Content-Length: ' . $fileSize);
+            header('Cache-Control: no-cache, must-revalidate, no-store, private');
+            header('Pragma: no-cache');
+            header('X-Content-Type-Options: nosniff');
+            header('X-Frame-Options: DENY');
+            header('X-XSS-Protection: 1; mode=block');
+
+            // Stream the local file with connection checks
+            $bufferSize = ($fileSize > 100 * 1024 * 1024) ? 1024 * 1024 : 8192; // 1MB for large files
+            while (!feof($fp) && connection_status() === CONNECTION_NORMAL) {
+                $chunk = fread($fp, $bufferSize);
+                if ($chunk === false) {
+                    break;
+                }
+                echo $chunk;
+                flush();
+            }
+
+            fclose($fp);
+
+            // Check if download was interrupted
+            if (connection_status() !== CONNECTION_NORMAL) {
+                error_log("Download interrupted - User: {$user}, Backup ID: {$backupId}");
+            } else {
+                error_log("AUDIT: Download completed - User: {$user}, Backup ID: {$backupId}, Method: local");
+            }
+
+        } else {
+            // Serve from remote storage
+            $storage = new BackupStorage($target->getCredentials());
+
+            // Get backup info
+            $info = $storage->getFileInfo($data['backup_path']);
+            if (!$info['success'] || !$info['exists']) {
+                error_log("Download failed - Remote file not found - User: {$user}, Backup ID: {$backupId}");
+                throw new Exception("Backup file is not currently available for download");
+            }
+
+            // Log successful remote file access
+            error_log("AUDIT: Serving backup from remote storage - User: {$user}, Backup ID: {$backupId}, Size: {$info['size']}");
+
+            // Set security headers
+            header('Content-Type: application/octet-stream');
+            header("Content-Disposition: attachment; filename=\"{$filename}\"; filename*=UTF-8''{$encodedFilename}");
+            header('Content-Length: ' . $info['size']);
+            header('Cache-Control: no-cache, must-revalidate, no-store, private');
+            header('Pragma: no-cache');
+            header('X-Content-Type-Options: nosniff');
+            header('X-Frame-Options: DENY');
+            header('X-XSS-Protection: 1; mode=block');
+
+            // Stream the file directly to output
+            $result = $storage->streamFile($data['backup_path']);
+
+            if (!$result['success']) {
+                error_log("Download failed - Remote stream error - User: {$user}, Backup ID: {$backupId}, Error: {$result['error']}");
+                throw new Exception("Unable to process backup download at this time");
+            }
+
+            error_log("AUDIT: Download completed - User: {$user}, Backup ID: {$backupId}, Method: remote");
         }
-        
+
         return true;
     }
     
